/*

 Modified for DIS Ignition test board by Juha Niinikoski / 2002

code adaptation 7.4.2002
unsigned int output 8.4.2002
temperature output 9.4.2002
kbhit & clear serial input functions added 11.4.2002
binary printout & get2hex added 11.4.2002
leading zero logic for temp printout 12.4.2002
get2hex echo added 10.10.2002


*/

#include	<pic18.h>
#include	<conio.h>
#include	<stdio.h>

void serial_setup(void)
{


/* Comms setup: */

	#define BAUD 9600
	#define DIVIDER ((PIC_CLK/(16UL * BAUD) -1))
	#define HIGH_SPEED 1


	//you can comment these #assert statements out if you dont want error checking
	#if PIC_CLK==3686400 && BAUD==19200
		#assert DIVIDER==11
	#elif PIC_CLK==4000000 && BAUD==19200
		#assert DIVIDER==12
	#elif PIC_CLK==16000000 && BAUD==19200
		#assert DIVIDER==51
	#elif PIC_CLK==20000000 && BAUD==19200
		#assert DIVIDER==64
	#endif

	SPBRG=DIVIDER;
	BRGH=HIGH_SPEED;	//data rate for sending
	SYNC=0;			//asynchronous
	SPEN=1;			//enable serial port pins
	CREN=1;			//enable reception
	SREN=0;			//no effect
	TXIE=0;			//disable tx interrupts
	RCIE=0;			//disable rx interrupts
	TX9=0;			//8-bit transmission
	RX9=0;			//8-bit reception
	TXEN=0;			//reset transmitter
	TXEN=1;			//enable the transmitter

	CTS = 0;		//enable host transmission

}

unsigned char dummy;


#define clear_usart_errors_inline	\
if (OERR)				\
{					\
	TXEN=0;				\
	TXEN=1;				\
	CREN=0;				\
	CREN=1;				\
}					\
if (FERR)				\
{					\
	dummy=RCREG;			\
	TXEN=0;				\
	TXEN=1;				\
}


/* writes a character to sekected output device */

void putch(unsigned char chr)
{
		while(!TXIF)				//set when register is empty
			{
			clear_usart_errors_inline;
			}
		TXREG=chr;
}


/* gets a character from the serial port */

unsigned char getch(void)
{

	while(!RCIF)
	{
		clear_usart_errors_inline;
	}

	return RCREG;
}


/* convert character to HEX nibble */

char cvt_hex(unsigned char ch)
{
	if (ch < '0')
		return(127);
	if (ch <= '9')
		return(ch - '0');

	if (ch < 'A')
		return (127);
	if (ch <= 'F')
		return(ch - 'A' + 10);

	if (ch < 'a')
		return (127);
	if (ch <= 'f')
		return(ch - 'a' + 10);
	return(127);
}

/* gets byte (2 hex characters) */

unsigned char get2hex()
{
	unsigned char temp;
	char input, echo;

	temp = 0;

		do
		{
		echo = getch();
		putch(echo);
		input = cvt_hex(echo);
		if (input == 127)
			putch(0x07);
		}
		while (input == 127);

	temp = input;

		do
		{
		echo = getch(); 
		putch(echo);
		input = cvt_hex(echo);
		if (input == 127)
			putch(0x07);
		}
		while (input == 127);

	temp = temp * 16 + input;

	return(temp);
}



/* Get unsigned int 0 - 65535 (1 - 5 digits terminated with ret) */

unsigned int getuint()
	{
unsigned int temp;
char inchar, digits;
	inchar = 0;
	temp = 0;
	digits = 5;

	while((inchar != 0x0D) && digits > 0)
		{
		inchar = getch();
			if((inchar >= 0x30) && (inchar <= 0x39))	// accept numbers only
				{
				temp = temp * 10;

				if(temp > (65535 - (inchar & 0x0F)))	// prevent overflow this round
					{
					putch('0');			// print last zero before leave input
					break;
					}

				digits--;				// max 5 digits
				putch(inchar);				// echo
				temp = temp + (inchar & 0x0F);		// add new entry

				if(temp > 6553)				// prevent overflow in next round
					break;
				}
		}
	return temp;
	}


/* check if character available, but do not read character serial port */
/* returns 0 if no characters available. Returns 1 if character available. */

unsigned char keypressed(void)
{
	if(RCIF)
	return(1);
	else
	return(0);
}

/* clear serial port receiver */

void clear_usart_input(void)
{
	clear_usart_errors_inline;
	dummy = RCREG;
}



void clear_usart_errors(void)
{
	clear_usart_errors_inline;
}




/* write character in hex format */

void putchhex(unsigned char chr)
{
	unsigned char temp;

	// transmits in hex

	temp=chr;

	chr=(chr >> 4);
	if (chr<10) chr+=48; else chr+=55;
	putch(chr);

	chr=temp;

	chr=(chr & 0x0F);
	if (chr<10) chr+=48; else chr+=55;
	putch(chr);
}


/* write unsigned in in hex format */

void putinthex(unsigned int chr)
{
	#define ramuint(x)				(*((unsigned int *) (x)))
	#define ramuint_hibyte(x)			(*(((unsigned char *)&x)+1))
	#define ramuint_lobyte(x)			(*(((unsigned char *)&x)+0))
	#define ramuchar(x)				(*((unsigned char *) (x)))

	putchhex(ramuint_hibyte(chr));
	putchhex(ramuint_lobyte(chr));

	#undef ramuint(x)
	#undef ramuint_hibyte(x)
	#undef ramuint_lobyte(x)
	#undef ramuchar(x)
}


/* write long in hex format */

void plong( long ww )
	{

	union
	{
	long w;
	unsigned char p[4];
	} lrek;

	lrek.w = ww;
	putchhex(lrek.p[3]);
	putchhex(lrek.p[2]);
	putchhex(lrek.p[1]);
	putchhex(lrek.p[0]);
	}


/* write character in decimal format, no negative numbers */
/* only hundreds digit leading zero suppress */

void putchdec(unsigned char chr)
{
	unsigned char temp;

	temp=chr;
	//hundreds
	if ((chr/100)>0) putch((chr/100)+'0');
	chr-=(chr/100)*100;

	//tens
	putch((chr/10)+'0');
	chr-=(chr/10)*10;

	//ones
	putch((chr/1)+'0');
}



/* writes unsigned int to serial port in decimal */
/* 5 digits, no leading zero suppression */

void putuintdec(unsigned int chr)
{
	//ten thousands
	putch((chr/10000)+'0');
	chr-=(chr/10000)*10000;

	//thousands
	putch((chr/1000)+'0');
	chr-=(chr/1000)*1000;

	//hundreds
	 putch((chr/100)+'0');
	chr-=(chr/100)*100;

	//tens
	putch((chr/10)+'0');
	chr-=(chr/10)*10;

	//ones
	putch((chr/1)+'0');

}


/* writes a character to the serial port in binary */

void putchbin(unsigned char ch)
{
	unsigned char x;

	for(x=0; x < 8; x++)
	{
	if (ch & 0x80)
		putch('1');
	else
		putch('0');
	ch = ch << 1;
	}

}

/* put string to the serial port */

void putst(register const char *str)
{
	while((*str)!=0)
	{
		putch(*str);
    if (*str==13) putch(10);
    if (*str==10) putch(13);
		str++;
	}
}

