/*

Simple wait and delay routines using Timer 1

Juha Niinikoski 10.10.2002

TMR2 IRQ based wait added 15.01.2002
Play engine 07.12.2004
Interrupt logic improved 10.12.2004


*/

// delays calculated for 20 MHz clock


/* 1mS wait counters */

static near int play_timer, wait_timer;


/* Init timers & interrupt system */

void Init_timers()
	{
	play_timer = 0;
	wait_timer = 0;
	play_flag = 0;
	tone_on = 0;

	PR2 = 249;		// Generate 1 ms IRQ. 20 MHz / 4 = 5 MHz / 4 / 250 / 5 = 1 mS
	T2CON = 0x21;		// Prescaler = 4 = 1250kHz clock for TMR2, postscaler = 5
	TMR2ON = 1;		// start TIMER2
	TMR2IE = 1;		// enable TMR2 IRQ

	RD16 = 1;		// Timer1 16-bit I/O mode
	TMR1IE = 1;
	TMR1ON = 1;

	GIEL = 1;		// enable low priority IRQs
	GIEH = 1;		// enable high priority IRQs
	}


void tmr1_int(void)				// play sound
	{
	TMR1IF = 0;				// reset IRQ flag
	
	TMR1 = tone;				// arm next half cycle

	if(tone_on)
		TONE = !TONE;			// output tone signal
	}


void tmr2_int(void)
	{
	TMR2IF = 0;				// reset IRQ flag

	if(play_timer != 0)			// decrement play timer
		{
		play_timer--;
		if(play_timer == 0)
			{
			tone_on = 0;		// stop sound
			wait_timer = STAGATO;	// arm note played timer
			}
		}

	if(wait_timer != 0)			// test wait timer
		{
		wait_timer--;			// decrement wait timer
		if(wait_timer == 0)
			{
			play_flag = 0;		// not playing, ready for next note
			LEDC = 0;		// play indicator off
			}
		}

	if(tout != 0)				// keyboard timeout
		{
		tout--;
		if(tout == 0)
			timeout = 1;
		}

	}

/* IRQ handlers */

//static void interrupt low_priority tim_int(void) // low priority

static void interrupt tim_int(void) 	// low priority
	{
	LEDB = 1;			// irq signal

	if(TMR2IF)			// check if TMR2 (1ms) int
		tmr2_int();

	if(TMR1IF)			// check if tone timer
		tmr1_int();

	LEDB = 0;			// irq signal

	}



/* Timer3 is used for uS delay and wait generation */
/* Set wait. Max wait (65535) 52400 uS, TMR3IF = expiration */

void set_timeout_u( int ticks )	
	{
	union
		{
		int t1;
		unsigned char t1regs[2];
		} timex;

	timex.t1 = 0xFFFF - ticks;
	T3CKPS1 = 1;			// prescaler = 4 ==> TMR1 Clock = 1,25 MHz (800ns tick)
	TMR3ON = 0;			// Set up TMR1 for time expired operation
	TMR3IF = 0;
	TMR3L = timex.t1regs[0];
	TMR3H = timex.t1regs[1];
	TMR3ON = 1;
	}


/* Test uS timeout */

BOOL is_timeout_u( void )
	{
	BOOL retval;
	retval = TMR3IF;
	return retval;
	}

/* Microsecond delay, small delays not accurate */
/* Overdead added to requested delay about 12uS */

void delayu( unsigned unsigned int usec )
{
	set_timeout_u( usec );
	while(!TMR3IF);
}


/* Millisecond delays */

void delaym( unsigned unsigned int millisec )
{
	while(millisec != 0)
		{
		delayu(1000);
		millisec --;
		}
}	


/* Play note */
/* t = note divider. If divider == 0 then play pause */
/* len = note play time. If len == 0 then play forever */
/* play_flag = 1 when playing */

void play( unsigned int t, unsigned int len )
	{
	play_flag = 1;			// playing state
	LEDC = 1;			// play indicator on
	if(t == 0)			// if pause
		{
		tone_on = 0;
		GIEH = 0;		// disable IRQs when doing timer & tone uppdate
		wait_timer = len;	// wait without sound
		tone = 0xFFFF - (kHz >> 1); // set 1kHz
		GIEH = 1;
		return;
		}

	GIEH = 0;			// disable IRQs when doing tone uppdate
	tone = 0xFFFF - (t >> 1);	// set tone pitch
	GIEH = 1;

	if(len != 0)			// if len == 0 then play forever
		{
		GIEH = 0;		// disable IRQs when doing timer uppdate	
		play_timer = len - STAGATO;	// start note timer
		GIEH = 1;
		}

	tone_on = 1;			// enable output
	}

void stop_play( void )			// stop playing note
	{
	play_flag = 0;
	LEDC = 0;
	tone_on = 0;
	GIEH = 0;
	tone = 0xFFFF - (kHz >> 1); 	// set 1kHz
	GIEH = 1;
	}
